@@ -1,4 +1,4 @@
-/* $Id: cdb_hash.c,v 1.2 2001/12/09 21:18:02 mjt Exp $
+/* $Id: cdb_hash.c,v 1.5 2003/11/03 16:42:41 mjt Exp $
  * cdb hashing routine
  *
  * This file is a part of tinycdb package by Michael Tokarev, mjt@corpit.ru.
@@ -7,12 +7,12 @@
 
 #include "cdb.h"
 
-cdbi_t
-cdb_hash(const void *buf, cdbi_t len)
+unsigned
+cdb_hash(const void *buf, unsigned len)
 {
   register const unsigned char *p = (const unsigned char *)buf;
   register const unsigned char *end = p + len;
-  register cdbi_t hash = 5381;	/* start value */
+  register unsigned hash = 5381;	/* start value */
   while (p < end)
     hash = (hash + (hash << 5)) ^ *p++;
   return hash;
