@@ -1,4 +1,4 @@
-/* $Id: cdb_unpack.c,v 1.2 2001/12/09 21:18:02 mjt Exp $
+/* $Id: cdb_unpack.c,v 1.5 2003/11/03 16:42:41 mjt Exp $
  * unpack 32bit integer
  *
  * This file is a part of tinycdb package by Michael Tokarev, mjt@corpit.ru.
@@ -7,10 +7,10 @@
 
 #include "cdb.h"
 
-cdbi_t
+unsigned
 cdb_unpack(const unsigned char buf[4])
 {
-  cdbi_t n = buf[3];
+  unsigned n = buf[3];
   n <<= 8; n |= buf[2];
   n <<= 8; n |= buf[1];
   n <<= 8; n |= buf[0];
